from org.python.modules import re

ITCAM_DC_HOME_VARIABLE = 'ITCAMDCHOME'
AM_HOME_PROPERTY = 'am.home'
JAVA_SECURITY_POLICY = 'java.security.policy'
DATACOLLECTOR_POLICY_FILE = '/datacollector.policy'
PROBE_INJECTOR_MANAGER_CHAIN = 'com.ibm.tivoli.jiti.injector.ProbeInjectorManagerChain.primaryInjectorFile'
PROBE_INJECTOR_MANAGER_CHAIN_FILE = '/probeinjectormanagerchain.properties'

def exit (exitCode):
    if exitCode > 0 :
    	print 'ITCAM-ERROR ' + exitCode
    else :
    	print 'ITCAM-SUCCESS'
    sys.exit(exitCode)


def usage():
    print 'Usage: nodeName serverName oldDCPath newDCPath libPathName runtimeName OSName oldLib newLib'


def getNodeID(nodeName, serverName) :
    print 'INFO: getNodeID start: ' +  nodeName + ', ' + serverName
    scope = '/Node:' + nodeName + '/Server:' + serverName + '/'
    nodeID = AdminConfig.getid('/Node:' + nodeName + '/Server:' + serverName + '/')
    print 'INFO: getNodeID complete: ' + nodeID
    return nodeID
    

def updateServerVariable(nodeID, varName, varValue) :
    varValue = varValue.replace('\\', '/')
    print 'INFO: updateVariable start: ' + nodeID + ', ' + varName + ', ' + varValue
    varSubstitutions = AdminConfig.list('VariableSubstitutionEntry',nodeID).split(java.lang.System.getProperty('line.separator'))
    for varSubst in varSubstitutions:
        getVarName = AdminConfig.showAttribute(varSubst, 'symbolicName')
        if getVarName == varName:
            AdminConfig.modify(varSubst,[['value', varValue]])
            print 'INFO: ServerVariable: ' + varName + ' is updated/created with value: ' + varValue
            break
    print 'INFO: updateVariable complete'


def updateSystemProperty(nodeID, varName, varValue) :
    varValue = varValue.replace('\\', '/')
    print 'INFO: updateSystemProperty start: ' + nodeID + ', ' + varName + ', ' + varValue
    jvm = AdminConfig.list('JavaVirtualMachine', nodeID)
    prop =[]
    props = splitProps(AdminConfig.showAttribute(jvm, "systemProperties"))
    for prop in props[:]:
        currName = AdminConfig.showAttribute(prop, "name")
        if currName == varName:
            AdminConfig.remove(prop)
            attrs = [["name", varName], ["value", varValue]]
            AdminConfig.create("Property", jvm, attrs, "systemProperties")
            print 'INFO: SystemProperty: ' + varName + ' is updated/created with value: ' + varValue
    print 'INFO: updateSystemProperty complete'


def splitProps(propString):
    propList = []
    tempProp = ''
    inQuotes = 0
    for char in propString[1:-1]:
        if char == ' ':
            if inQuotes == 0:
                propList.append(tempProp)
                tempProp = ''
            else:
                tempProp = tempProp + char
        elif char == '"':
            if inQuotes == 0:
                inQuotes = 1
            else:
                inQuotes = 0
        else:
            tempProp = tempProp + char
    if tempProp != '':
        propList.append(tempProp)
    return propList


def updateLibPath(nodeID, varName, oldValue, newValue) :
    print 'INFO: updateLibPath start: ' + nodeID + ', ' +  varName + ', ' + oldValue + ', ' + newValue
    try:
        jpd = AdminConfig.list('JavaProcessDef', nodeID)
        prop =[]
        props = splitProps(AdminConfig.showAttribute(jpd, "environment"))
        for prop in props[:]:
            currName = AdminConfig.showAttribute(prop, "name")
            oldVL = AdminConfig.showAttribute(prop, "value")
            print currName + ' = ' + oldVL
            if currName == varName:
                newVL = replaceDependsOS(oldVL, oldValue, newValue)
                AdminConfig.remove(prop)
                attrs = [["name", varName], ["value", newVL]]
                AdminConfig.create("Property", jpd, attrs, "environment")
                print 'INFO: Library path variable: ' + varName + ' is updated to value: ' + newValue 
    except Exception :
       print 'Error: An error occured in updateLibPath'
       exit(4)
    print 'INFO: updateLibPath complete'


def updateGenericJVMArg(nodeID, oldValue, newValue) :
    print 'INFO: updateGenericJVMArg start: ' + nodeID + ', ' + oldValue + ', ' + newValue
    try :
        jvm = AdminConfig.list('JavaVirtualMachine', nodeID)
        oldVL = AdminConfig.showAttribute(jvm, 'genericJvmArguments')
        newVL = replaceDependsOS(oldVL, oldValue, newValue)
        args='[[genericJvmArguments "'+newVL+'"]]'
        AdminConfig.modify(jvm, args)
        print 'INFO: Profiler option is updated to value: ' + newValue
    except Exception :
       print 'Error: An error occured in updateGenericJVMArg'
       exit(3)    
    print 'INFO: updateGenericJVMArg complete'


def sync(nodeName) :
    print 'INFO: sync start: ' + nodeName
    try :
        cellID = AdminConfig.getid('/Cell:/')
        cellType = AdminConfig.showAttribute(cellID, "cellType")
        if cellType != 'STANDALONE':
            Sync = AdminControl.completeObjectName('type=NodeSync,node=' + nodeName + ',*')
            if Sync != '' :
                result = AdminControl.invoke(Sync, 'sync')
                if result == 'true':
                    print "Info: configuration was changed and synchronized with node."
                    return
                else:
                    print "Warning: configuration was changed but not synchronized with node."
            else:
                print "Warning: configuration was changed but not synchronized with node (node is down)."
    except Exception :
        print "Warning: configuration was changed but not synchronized with node - exception occured."
    print 'INFO: sync complete'


def replaceDependsOS(content, oldValue, newValue) :
    print 'INFO: replaceDependsOS start: ' + content + ', ' + oldValue + ', ' + newValue
    if OSName == 'windows' :
        newValue = newValue.replace('\\', '/')
        oldValue = re.sub(r'(\\\\)+|/+', '((\\\\\\\\)+|/+)', oldValue)
    if OSName == 'windows' or OSName == 'os400' :
    	for i in range(ord('A'), ord('Z')+1) :
            re1 = r"("+chr(i)+"|"+(chr(ord(i)+(ord('a')-ord('A')))+")")
            re2 = r"("+chr(i)+"|"+(chr(ord(i)+(ord('a')-ord('A')))+")")
            oldValue = re.sub(re1, re2, oldValue)
    result = re.sub(oldValue, newValue, content)
    result = re.sub(r'(\\)+','/',result)
    print 'INFO: replaceDependsOS complete: ' + result
    return result


#main
print 'INFO: updateDCPath.py start'
if len(sys.argv) < 9 :
    print 'Wrong arguments count'
    usage()
    exit(2)

# get arguments
nodeName     = sys.argv[0]
serverName   = sys.argv[1]
oldDCPath    = sys.argv[2]
newDCPath    = sys.argv[3]
libPathName  = sys.argv[4]
runtimeName  = sys.argv[5]
OSName       = sys.argv[6]
oldLib       = sys.argv[7]
newLib       = sys.argv[8]

# init work variables
amHome     = newDCPath + '/itcamdc'
newRuntime  = newDCPath + '/runtime/' + runtimeName

try:
    nodeID = getNodeID(nodeName, serverName)

    updateServerVariable(nodeID, ITCAM_DC_HOME_VARIABLE, newDCPath)

    updateSystemProperty(nodeID, AM_HOME_PROPERTY, amHome)
    updateSystemProperty(nodeID, JAVA_SECURITY_POLICY, newRuntime + DATACOLLECTOR_POLICY_FILE)    
    updateSystemProperty(nodeID, PROBE_INJECTOR_MANAGER_CHAIN, newRuntime + PROBE_INJECTOR_MANAGER_CHAIN_FILE)

    updateLibPath(nodeID, libPathName, oldDCPath, newDCPath)

    updateGenericJVMArg(nodeID, oldDCPath, newDCPath)
    
    if OSName == 'os400' :
    	updateGenericJVMArg(nodeID, oldLib, newLib)    
    
except Exception :
    print 'Error: An error occured'
    exit(1)
else:
    AdminConfig.save()
    sync(nodeName)
    exit(0)
